@@ -5,38 +5,71 @@
 //              with the application.
 // ---------------------------------------------------------------------------
 // $Source: src/lib.rs $
-// $Revision: 2021-12-19T21:17:34+01:00 $
+// $Revision: 2022-01-04T02:42:37+01:00 $
 
+#![doc = include_str ! ("../README.md")]
 #![cfg_attr(not(feature = "std"), no_std)]
 #![allow(dead_code)]
 
+use core::cmp::Ordering;
 use core::fmt;
 use core::ops::{Add, Div, Mul, Sub};
 #[cfg(fpdec)]
 use fpdec::{Dec, Decimal};
 pub use si_prefixes::SIPrefix;
-pub use unitless::{Unitless, NONUNIT};
+pub use unitless::{Unitless, ONE};
 
-mod macros;
+pub mod prelude;
 mod si_prefixes;
 mod unitless;
 
 #[cfg(fpdec)]
+/// Type used for the numerical part of a Quantity.
 pub type AmountT = Decimal;
 #[cfg(all(not(fpdec), target_pointer_width = "64"))]
+/// Type used for the numerical part of a Quantity.
 pub type AmountT = f64;
 #[cfg(all(not(fpdec), target_pointer_width = "32"))]
+/// Type used for the numerical part of a Quantity.
 pub type AmountT = f32;
 
-pub trait Unit: Copy + PartialEq + Sized + Mul<AmountT> {
+#[cfg(fpdec)]
+#[allow(non_snake_case)]
+#[macro_export]
+/// Converts a numerical literal to an AmountT.
+macro_rules! Amnt {
+    ($lit:literal) => {
+        Dec!($lit)
+    };
+}
+#[cfg(not(fpdec))]
+#[allow(non_snake_case)]
+#[macro_export]
+/// Converts a numerical literal to an AmountT.
+macro_rules! Amnt {
+    ($lit:literal) => {
+        $lit
+    };
+}
+
+/// The abstract type of units used to define quantities.
+pub trait Unit: Copy + Eq + PartialEq + Sized + Mul<AmountT> {
+    /// Optional unit used as reference for scaling the units.
     const REF_UNIT: Option<Self>;
 
+    /// Returns an iterator over the variants of `Self`.
+    fn iter<'a>() -> core::slice::Iter<'a, Self>;
+
+    /// Returns the name of `self`.
     fn name(&self) -> &'static str;
 
+    /// Returns the symbol used to represent `self`.
     fn symbol(&self) -> &'static str;
 
+    /// Returns the SI prefix of `self`, or None is `self` is not a SI unit.
     fn si_prefix(&self) -> Option<SIPrefix>;
 
+    /// Returns `true` if `self` is the reference unit of its unit type.
     #[inline]
     fn is_ref_unit(&self) -> bool {
         Self::REF_UNIT == Some(*self)
@@ -56,9 +89,15 @@ pub trait Unit: Copy + PartialEq + Sized + Mul<AmountT> {
     }
 }
 
+/// The abstract type of quantities.
 pub trait Quantity<U: Unit>:
     Copy + Sized + Add<Self> + Sub<Self> + Div<Self> + Mul<AmountT>
 {
+    /// Returns an iterator over the variants of `U`.
+    fn iter_units<'a>() -> core::slice::Iter<'a, U> {
+        U::iter()
+    }
+
     /// Returns a new instance of `Quantity<U>`.
     fn new(amount: AmountT, unit: U) -> Self;
 
@@ -87,6 +126,7 @@ pub trait Quantity<U: Unit>:
 }
 
 #[derive(Copy, Clone, Debug)]
+/// Generic struct that implements trait `Quantity`.
 pub struct Qty<U: Unit> {
     amount: AmountT,
     unit: U,
@@ -118,6 +158,34 @@ impl<U: Unit> Quantity<U> for Qty<U> {
     }
 }
 
+impl<U: Unit> PartialEq for Qty<U> {
+    fn eq(&self, other: &Self) -> bool {
+        if self.unit == other.unit {
+            self.amount == other.amount
+        } else {
+            match other.equiv_amount(self.unit) {
+                None => false,
+                Some(equiv_amount) => self.amount == equiv_amount,
+            }
+        }
+    }
+}
+
+impl<U: Unit> Eq for Qty<U> {}
+
+impl<U: Unit> PartialOrd for Qty<U> {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        if self.unit == other.unit {
+            self.amount().partial_cmp(&other.amount())
+        } else {
+            match self.equiv_amount(other.unit) {
+                None => None,
+                Some(equiv_amount) => equiv_amount.partial_cmp(&other.amount()),
+            }
+        }
+    }
+}
+
 impl<U: Unit> Add<Self> for Qty<U> {
     type Output = Self;
 
@@ -145,7 +213,7 @@ impl<U: Unit> Div<Self> for Qty<U> {
 
     fn div(self, rhs: Self) -> Self::Output {
         match rhs.equiv_amount(self.unit) {
-            Some(equiv) => Self::Output::new(self.amount / equiv, NONUNIT),
+            Some(equiv) => Self::Output::new(self.amount / equiv, ONE),
             None => panic!("Incompatible units!"),
         }
     }
